/**
 * @Author: Jet.Chan
 * @Date:   2016-05-30T11:41:36+08:00
 * @Email:  guanjie.chen@talebase.com
* @Last modified by:   Jet.Chan
* @Last modified time: 2017-02-17T11:56:35+08:00
 */



/*
    日期选择框
*/
'use strict';

require('./style.less');

var $ = require('jquery');
var stapes = require('stapes');

var tpl = [
    '<div class="cui-datePicker">' +
    '<div class="cui-header">' +
    '<a class="prev">' +
    '<i class="icon-component cpf-icon-ic_prev"></i>' +
    '</a>' +
    '<a class="month"></a>' +
    '<a class="year"></a>' +
    '<a class="next">' +
    '<i class="icon-component cpf-icon-ic_next"></i>' +
    '</a>' +
    '</div>' +
    '<div class="cui-inner"></div>'
].join('');

var datePicker = stapes.subclass({
    constructor: function($el) {
        var self = this;
        if (!$el) {
            return;
        }

        self.$el = $el;
        self.$tpl = $(tpl);
        self.val = null;
        self.current = new Date();
        self.mode = 'day';
        self.init();
    },
    init: function() {
        var self = this;

        self.$el.append(self.$tpl);

        self.setWeek();
        self.setCurrent();

        self.watch();
    },
    watch: function() {
        var self = this;
        self.$el.on('click', function(e) {
            clearTimeout(self.timeout);
            if (!self.$el.hasClass('active')) {
                self.open();

                //clickaway
                e.stopPropagation();
                //self.$el.parents('body').off('click');
                self.$el.parents('body').one('click', function() {
                    clearTimeout(self.timeout);
                    self.close();
                });
            } else {
                self.close();
            }
        });

        self.$el.find('.cui-datePicker').on('click', function(e) {
            e.stopPropagation();
        });

        self.$el.find('.cui-datePicker .cui-header .next').one('click', function() {
            self.next();
        });

        self.$el.find('.cui-datePicker .cui-header .prev').one('click', function() {
            self.prev();
        });

        self.$el.find('.cui-datePicker .cui-header .month').on('click', function() {
            self.mode = 'month';
            self.$el.find('.cui-datePicker .cui-inner').children().fadeOut(150, function() {
                self.$el.find('.cui-datePicker .cui-inner').empty();
                self.setCurrentMonth();
            });
        });

        self.$el.find('.cui-datePicker .cui-header .year').on('click', function() {
            self.mode = 'year';
            self.$el.find('.cui-datePicker .cui-inner').children().fadeOut(150, function() {
                self.$el.find('.cui-datePicker .cui-inner').empty();
                self.setCurrentYear();
            });
        });
    },
    open: function() {
        var self = this;
        self.$el.find('.cui-datePicker').css('display', 'block');
        self.timeout = setTimeout(function() {
            self.$el.addClass('active');
        }, 0);
    },
    close: function() {
        var self = this;
        self.$el.removeClass('active');
        self.timeout = setTimeout(function() {
            self.$el.find('.cui-datePicker').css('display', 'none');
        }, 300);
    },
    next: function() {
        var self = this;

        switch (self.mode) {
            case 'day':
                var year = self.current.getFullYear();
                var month = self.current.getMonth();
                self.current = new Date(year, month + 1, 1);

                self.setCurrent();

                var middle = self.$el.find('.cui-datePicker .cui-inner .days').eq(0);
                var next = self.$el.find('.cui-datePicker .cui-inner .days').eq(1);
                next.addClass('days-next');

                middle.css({
                    position: 'absolute',
                    top: '26px'
                });
                next.css({
                    position: 'relative',
                    top: '0px'
                });

                break;
            case 'month':
                year = self.current.getFullYear() + 1;
                self.current = new Date(year, 1, 1);

                self.setCurrentMonth();

                middle = self.$el.find('.cui-datePicker .cui-inner .months').eq(0);
                next = self.$el.find('.cui-datePicker .cui-inner .months').eq(1);
                next.addClass('months-next');

                middle.css({
                    position: 'absolute'
                });
                next.css({
                    position: 'relative'
                });

                break;
            case 'year':
                year = self.current.getFullYear() + 25;
                self.current = new Date(year, 1, 1);

                self.setCurrentYear();

                middle = self.$el.find('.cui-datePicker .cui-inner .years').eq(0);
                next = self.$el.find('.cui-datePicker .cui-inner .years').eq(1);
                next.addClass('years-next');

                middle.css({
                    position: 'absolute'
                });
                next.css({
                    position: 'relative'
                });

                break;
            default:
                break;
        }

        middle.stop().animate({
            left: '-100%'
        }, 200, function() {
            $(this).remove();
        });
        next.stop().animate({
            left: '0'
        }, 200, function() {
            $(this).removeClass('days-next months-next');
            self.$el.find('.cui-datePicker .cui-header .next').one('click', function() {
                self.next();
            });
        });
    },
    prev: function() {
        var self = this;

        switch (self.mode) {
            case 'day':
                var year = self.current.getFullYear();
                var month = self.current.getMonth();
                self.current = new Date(year, month - 1, 1);

                self.setCurrent();

                var middle = self.$el.find('.cui-datePicker .cui-inner .days').eq(0);
                var prev = self.$el.find('.cui-datePicker .cui-inner .days').eq(1);
                prev.addClass('days-prev');

                middle.css({
                    position: 'absolute',
                    top: '26px'
                });
                prev.css({
                    position: 'relative',
                    top: '0px'
                });
                break;
            case 'month':
                year = self.current.getFullYear() - 1;
                self.current = new Date(year, 1, 1);

                self.setCurrentMonth();

                middle = self.$el.find('.cui-datePicker .cui-inner .months').eq(0);
                prev = self.$el.find('.cui-datePicker .cui-inner .months').eq(1);
                prev.addClass('months-prev');

                middle.css({
                    position: 'absolute'
                });
                prev.css({
                    position: 'relative'
                });
                break;
            case 'year':
                year = self.current.getFullYear() - 25;
                self.current = new Date(year, 1, 1);

                self.setCurrentYear();

                middle = self.$el.find('.cui-datePicker .cui-inner .years').eq(0);
                prev = self.$el.find('.cui-datePicker .cui-inner .years').eq(1);
                prev.addClass('years-prev');

                middle.css({
                    position: 'absolute'
                });
                prev.css({
                    position: 'relative'
                });
                break;
            default:
                break;
        }

        middle.stop().animate({
            left: '100%'
        }, 200, function() {
            $(this).remove();
        });
        prev.stop().animate({
            left: '0'
        }, 200, function() {
            $(this).removeClass('days-prev');
            self.$el.find('.cui-datePicker .cui-header .prev').one('click', function() {
                self.prev();
            });
        });
    },
    getValue: function() {
        var self = this;
        return self.val;
    },
    setValue: function(val) {
        var self = this;

        self.mode = 'day';

        self.current = val;
        self.$el.find('.cui-datePicker .cui-inner').empty();
        self.setWeek();
        self.setCurrent();
        //self.$el.find('.cui-datePicker .cui-inner .days').eq(0).remove();

        var _val = val.getFullYear() + '-' + (val.getMonth() + 1) + '-' + val.getDate();
        self.$el.find('button').map(function(i, v) {
            if ($(v).data('date') === _val) {
                self.setActive($(v));
                self.setResult($(v));
            }
        }, this);
    },
    setActive: function(item) {
        item.siblings().filter('button').removeClass('activeItem');
        item.addClass('activeItem');
    },
    setResult: function(item) {
        var self = this;
        var date = item.data('date');
        var ph = self.$el.find('.placeholder');
        if (ph.html()) {
            ph.removeClass('placeholder');
            ph.addClass('result');
            ph.text(date);
        } else {
            self.$el.find('.result').text(date);
        }
        self.val = new Date(date.split('-')[0], date.split('-')[1] - 1, date.split('-')[2]);
    },
    setWeek: function() {
        var self = this;

        var languageTarget = self.$el.data('language');
        var weekArray = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
        switch (languageTarget) {
            case 'en':
                weekArray = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
                break;
            case 'zh':
                weekArray = ['日', '一', '二', '三', '四', '五', '六']
                break;
            default:
        }
        var weeks = $('<div class="weeks">');
        weekArray.forEach(function(v) {
            var div = $('<div class="week">').text(v);
            weeks.append(div);
        });
        self.$el.find('.cui-datePicker .cui-inner').append(weeks);
    },
    getCurrent: function() {
        var self = this;
        var year = self.current.getFullYear(),
            month = self.current.getMonth(),
            first = new Date(year, month, 1),
            end = new Date(year, month + 1, 0),
            min = 1 - first.getDay(),
            max = (Math.ceil((end.getDate() - min + 1) / 7) * 7) + min - 1,
            dayArray = [];

        for (var day = min; day <= max; day++) {
            var date = new Date(year, month, day);
            dayArray.push(date);
        }
        return dayArray;
    },
    setCurrent: function() {
        var self = this;

        var languageTarget = self.$el.data('language');
        var monthArray = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
        switch (languageTarget) {
            case 'zh':
                monthArray = ['一月', '二月', '三月', '四月', '五月', '六月', '七月', '八月', '九月', '十月', '十一月', '十二月']
                break;
            default:
        }
        self.$tpl.find('.cui-header a.month').text(monthArray[self.current.getMonth()]);
        self.$tpl.find('.cui-header a.year').text(self.current.getFullYear());

        var dayArray = self.getCurrent();
        var days = $('<div class="days">');
        dayArray.forEach(function(v) {
            var button = $('<button class="day" data-date=' + v.getFullYear() + '-' + (v.getMonth() + 1) + '-' + v.getDate() + '>').text(v.getDate());
            if (v.getMonth() !== self.current.getMonth()) {
                button.addClass('gray');
            }
            if (v - self.val === 0) {
                self.setActive(button);
            }
            button.on('click', function() {
                self.setActive($(this));
                self.setResult($(this));
                self.close();
            });
            days.append(button);
        });
        self.$el.find('.cui-datePicker .cui-inner').append(days);
    },
    setCurrentMonth: function() {
        var self = this;

        self.$tpl.find('.cui-header a.year').text(self.current.getFullYear());

        var languageTarget = self.$el.data('language');
        var monthArray = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
        switch (languageTarget) {
            case 'zh':
                monthArray = ['一月', '二月', '三月', '四月', '五月', '六月', '七月', '八月', '九月', '十月', '十一月', '十二月']
                break;
            default:
        }
        var months = $('<div class="months">');
        monthArray.forEach(function(v, i) {
            var button = $('<button class="monthItem" data-month=' + i + '>').text(v);
            button.on('click', function() {
                self.mode = 'day';
                self.current = new Date(self.current.getFullYear(), i, 1);
                self.$el.find('.cui-datePicker .cui-inner').children().fadeOut(150, function() {
                    self.$el.find('.cui-datePicker .cui-inner').empty();
                    self.setWeek();
                    self.setCurrent();
                });
            });
            months.append(button);
        });
        self.$el.find('.cui-datePicker .cui-inner').append(months);
    },
    setCurrentYear: function() {
        var self = this;

        self.$tpl.find('.cui-header a.year').text(self.current.getFullYear());

        var yearArray = [];
        for (var i = -12; i <= 12; i++) {
            yearArray.push(self.current.getFullYear() + i);
        }
        var years = $('<div class="years">');
        yearArray.forEach(function(v) {
            var button = $('<button class="yearItem" data-year=' + v + '>').text(v);
            button.on('click', function() {
                self.mode = 'month';
                self.current = new Date(v, 1, 1);
                self.$el.find('.cui-datePicker .cui-inner').children().fadeOut(150, function() {
                    self.$el.find('.cui-datePicker .cui-inner').empty();
                    self.setCurrentMonth();
                });
            });
            years.append(button);
        });
        self.$el.find('.cui-datePicker .cui-inner').append(years);
    }
});

module.exports = datePicker;
