/*
    年月选择框
*/
'use strict';

require('./style.less');

var $ = require('jquery');
var stapes = require('stapes');

var util = require('../../../lib/util');

var tpl = [
    '<div class="cui-dateWrapper">' +
    '<hr/>' +
    '<div class="cui-date">' +
    '<div class="cui-years"></div>' +
    '<div class="cui-months"></div>' +
    '</div>' +
    '</div>'
].join('');

var dateSelector = stapes.subclass({
    constructor: function($el) {
        var self = this;
        //console.log($el);
        if (!$el) {
            return;
        }
        self.$el = $el;
        self.$tpl = $(tpl);
        self.timeout = null;
        self.val = null;
        self.year = '';
        self.month = '';
        self.init(new Date().getFullYear() + ($el.data('start') || 0));
    },
    init: function(yearStart) {
        var self = this;

        var ul = $('<ul>');
        for (var i = 0; i < 60; i++) {
            var li = $('<li class="year" data-year=' + (yearStart - i) + '>').text(yearStart - i);
            ul.append(li);
        }
        self.$tpl.find('.cui-years').append(ul);

        if (self.$el.data('lang') == 'en') {
            var monthArray = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
        } else {
            monthArray = ['1月', '2月', '3月', '4月', '5月', '6月', '7月', '8月', '9月', '10月', '11月', '12月'];
        }

        monthArray.map(function(v, k) {
            var div = $('<div class="month" data-month=' + k + '>').text(v);
            self.$tpl.find('.cui-months').append(div);
        }, this);

        if (self.$el.data('lang') == 'en') {
            self.$tpl.find('.cui-months .month').css('font-size', '14px');
        }

        self.$el.append(self.$tpl);

        self.watch();
    },
    watch: function() {
        var self = this;
        self.$el.on('click', function(e) {
            clearTimeout(self.timeout);
            if (self.$el.hasClass('active')) {
                self.close();
            } else {
                self.open();

                //clickaway
                e.stopPropagation();
                //self.$el.parents('body').off('click');
                self.$el.parents('body').one('click', function() {
                    clearTimeout(self.timeout);
                    self.close();
                });
            }
        });

        self.$el.find('.cui-date').on('click', function(e) {
            e.stopPropagation();
        });

        self.$el.find('.cui-years .year').on('click', function() {
            self.setActive($(this));
            self.year = $(this).data('year');
            if (self.month !== '') {
                self.setResult();
                //self.close();
            }
        });

        self.$el.find('.cui-months .month').on('click', function() {
            self.setActive($(this));
            self.month = $(this).data('month');
            if (self.year !== '') {
                self.setResult();
                //self.close();
            }
        });
    },
    close: function() {
        var self = this;
        self.$el.removeClass('active');
        if (util.ie <= 8) {
            self.$el.find('.cui-date').css('display', 'none');
            return;
        }
        self.timeout = setTimeout(function() {
            self.$el.find('.cui-date').css('display', 'none');
        }, 400);
    },
    open: function() {
        var self = this;
        self.$el.find('.cui-date').css('display', 'block');
        self.timeout = setTimeout(function() {
            self.$el.addClass('active');
        }, 0);
    },
    getValue: function() {
        var self = this;
        return self.val;
    },
    setValue: function(val) {
        var self = this;
        self.$el.find('.year').map(function(i, v) {
            if ($(v).data('year') === val.getFullYear()) {
                self.setActive($(v));
                self.year = parseInt(val.getFullYear(), 10);
            }
        }, this);
        self.$el.find('.month').map(function(i, v) {
            if ($(v).data('month') === val.getMonth()) {
                self.setActive($(v));
                self.month = parseInt(val.getMonth(), 10);
            }
        }, this);

        self.setResult();
    },
    setActive: function(item) {
        item.siblings().removeClass('activeItem');
        item.addClass('activeItem');
    },
    setResult: function() {
        var self = this;
        var year = self.year;
        var month = (self.month > 8 ? '' : '0') + (self.month + 1);

        var ao = self.$el.find('.addon');
        if (!ao.html()) {
            var result = year + '-' + month;
            var ph = self.$el.find('.placeholder');
            if (ph.html()) {
                ph.removeClass('placeholder');
                ph.addClass('result');
                ph.text(result);
            } else {
                self.$el.find('.result').text(result);
            }
        } else {
            result = year + '-' + month;
            var rs = self.$el.find('.result');
            if (rs.html()) {
                rs.text(result);
            } else {
                rs = $('<span class="result">');
                rs.text(result);
                rs.insertAfter(ao);
            }
        }
        self.val = new Date(self.year, self.month, 1);
    }
});

module.exports = dateSelector;
