/**
* @Author: Jet.Chan
* @Date:   2016-05-30T11:41:36+08:00
* @Email:  guanjie.chen@talebase.com
* @Last modified by:   Jet.Chan
* @Last modified time: 2017-02-22T11:37:45+08:00
*/



/*
    下拉选择框
*/
'use strict';

require('./style.less');

var $ = require('jquery');
var stapes = require('stapes');

var util = require('../../../lib/util');

var tpl = [
    '<div class="cui-optionsWrapper">' +
    '<hr/>' +
    '<div class="cui-options"></div>' +
    '</div>'
].join('');

var selectBox = stapes.subclass({
    constructor: function($el) {
        var self = this;
        //console.log($el);
        if (!$el || !$el.data('options')) {
            return;
        }
        self.$el = $el;
        self.data = {
            options: $el.data('options')
        };
        self.$tpl = $(tpl);
        self.timeout = null;
        self.val = null;
        self.init();
    },
    init: function() {
        var self = this;
        var ul = $('<ul>');
        self.data.options.forEach(function(v) {
            var li = $('<li data-value="' + v.value + '">').text(v.text);
            ul.append(li);
        });
        self.$tpl.find('.cui-options').append(ul);
        self.$el.append(self.$tpl);

        self.watch();
    },
    watch: function() {
        var self = this;
        self.$el.on('click', function(e) {
            clearTimeout(self.timeout);
            if (self.$el.hasClass('active')) {
                self.close();
            } else {
                self.open();

                //clickaway
                e.stopPropagation();
                //self.$el.parents('body').off('click');
                self.$el.parents('body').one('click', function() {
                    clearTimeout(self.timeout);
                    self.close();
                });
            }
        });

        self.$el.find('li').on('click', function() {
            self.setActive($(this));
            self.setResult($(this));
        });
    },
    close: function() {
        var self = this;
        self.$el.removeClass('active');
        if (util.ie <= 8) {
            self.$el.find('.cui-options').css('display', 'none');
            return;
        }
        self.timeout = setTimeout(function() {
            self.$el.find('.cui-options').css('display', 'none');
        }, 400);
    },
    open: function() {
        var self = this;
        self.$el.find('.cui-options').css('display', 'block');
        self.timeout = setTimeout(function() {
            self.$el.addClass('active');
        }, 0);
    },
    getValue: function() {
        var self = this;
        return self.val;
    },
    setValue: function(val) {
        var self = this;
        self.$el.find('li').map(function(i, v) {
            if ($(v).data('value') === val.value) {
                self.setActive($(v));
                self.setResult($(v));
            }
        }, this);
    },
    setActive: function(item) {
        item.siblings().removeClass('activeItem');
        item.addClass('activeItem');
    },
    setResult: function(item) {
        var self = this;

        var ao = self.$el.find('.addon');
        if (!ao.html()) {
            var ph = self.$el.find('.placeholder');
            if (ph.html()) {
                ph.removeClass('placeholder');
                ph.addClass('result');
                ph.text(item.text());
            } else {
                self.$el.find('.result').text(item.text());
            }
        } else {
            var rs = self.$el.find('.result');
            if (rs.html()) {
                rs.text(item.text());
            } else {
                rs = $('<span class="result">');
                rs.text(item.text());
                rs.insertAfter(ao);
            }
        }

        self.val = {
            value: item.data('value'),
            text: item.text()
        };
    }
});

module.exports = selectBox;
